<?php
/** 
 * Plugin Name: WP Basic Elements
 * Plugin URI: https://wordpress.org/plugins/wp-basic-elements/
 * Description: Disable unnecessary features and speed up your site. Make the WP Admin simple and clean.
 * Version: 5.2.11
 * Author: Damir Calusic
 * Author URI: https://www.damircalusic.com/
 * Text Domain: wp-basic-elements
 * Domain Path: /languages/
 * License: GPLv2
 * License URI: https://www.gnu.org/licenses/gpl-2.0.html
 */

// Exit if accessed directly
if(!defined('ABSPATH')) exit;

// Define global variables
define('WBE_VERSION', '5.2.11');

/**
* Check if the Class exists otherwise create it
*/
if(!class_exists('WPBE_Functions')){
    /**
     * Class WPBE_Functions
     */
    class WPBE_Functions {
        /**
         * The reference the WPBE_Functions instance of this class.
         *
         * @since 5.2.0
         * @var $instance
         */
        protected static $instance;

        /**
         * The refrence to the settings page url
         * 
         * @since 5.2.0
         * @string settings_page_endpoint
         */
        public $settings_page_endpoint = 'wpbe_settings_page';

        /**
         * Donate links
         * 
         * @since 5.2.7
         * @string $donate_link_paypal
         * @string $donate_link_patreon
         */
        public $donate_link_paypal = 'https://www.paypal.com/cgi-bin/webscr?cmd=_s-xclick&hosted_button_id=AJABLMWDF4RR8&source=url';
        public $donate_link_patreon = 'https://www.patreon.com/webkreativ';
       
        /**
         * Contains all the settings keys for the WP Core section
         *
         * @since 5.2.0
         * @array $wpcore
         */
        public $wpcore = [];

        /**
         * Contains all the settings keys for the WP Head section
         *
         * @since 5.2.0
         * @array $wphead
         */
        public $wphead = [];

        /**
         * Contains all the settings keys for the WP Dashboard section
         *
         * @since 5.2.0
         * @array $wpdashboard
         */
        public $wpdashboard = [];

        /**
         * Contains all the settings keys for the WP Admin Toolbar section
         *
         * @since 5.2.0
         * @array $wpadmintoolbar
         */
        public $wpadmintoolbar = [];

        /**
         * Contains all the settings keys for the WP Users section
         *
         * @since 5.2.0
         * @array $wpusers
         */
        public $wpusers = [];

        /**
         * Contains all the settings keys for the WP Admin Footer section
         *
         * @since 5.2.0
         * @array $wpadminfooter
         */
        public $wpadminfooter = [];

        /**
         * Contains all the settings keys for the WP Mail section
         *
         * @since 5.2.0
         * @array $wpmail
         */
        public $wpmail = [];

        /**
         * Contains all the settings keys for the Gutenberg section
         *
         * @since 5.2.0
         * @array $gutenberg
         */
        public $gutenberg = [];    

        /**
         * Contains all the settings keys for the Plugins section
         *
         * @since 5.2.0
         * @array $plugins
         */
        public $plugins = [];
        
        /**
         * Contains all unserialized settings
         * 
         * @since 5.2.0
         * @array $wpbe_settings
         */
        public $wpbe_settings = [];

        /**
         * Contains the proper format of settings
         * 
         * @since 5.2.0
         * @array $wpbe_keys
         */
        public $wpbe_keys = [];

        /**
         * Contains WP Editor settings
         * 
         * @since 5.2.3
         * @array $wp_editor_settings
         */
        public $wp_editor_settings = array('media_buttons' => false, 'quicktags' => false, 'tinymce' => array('toolbar1' => 'bold,italic,underline,link,unlink', 'toolbar2' => '', 'toolbar3' => ''));

        /**
         * Protected constructor to prevent creating a new instance of the WPBE_Functions via the `new` operator from outside of this class.
         */
        protected function __construct() {
            add_action('plugins_loaded', array($this, 'wpbe_plugins_loaded'));
            $this->init_wpbe_functions();
        }

        /**
         * Returns the WPBE_Functions instance of this class.
         *
         * @since 5.2.0
         * @return self::$instance The WPBE_Functions instance.
         */
        public static function get_instance(){
            if(null === self::$instance){
                self::$instance = new self();
            }

            return self::$instance;
        }

        /**
         * Initiate the WPBE Plugin
         * 
         * @since 5.2.0
         */
        public function init_wpbe_functions(){
            // Run the important Admin functions
            add_action('admin_enqueue_scripts', array($this, 'wpbe_admin_enqueue_scripts'));
            add_action('admin_menu', array($this, 'wpbe_admin_menu'));
            add_action('wp_ajax_wpbe_save_settings', array($this, 'wpbe_save_settings'));

            // Execute the needed settings checks on frontend and backend
            add_action('init', array($this, 'wpbe_init'));
            add_action('wp_before_admin_bar_render', array($this,'wpbe_wp_before_admin_bar_render'));
            add_action('wp_dashboard_setup', array($this,'wpbe_wp_dashboard_setup'));
            add_action('wp_enqueue_scripts', array($this,'wpbe_wp_enqueue_scripts'), 999);
            add_action('enqueue_block_editor_assets', array($this,'wpbe_enqueue_block_editor_assets'));
            add_filter('wp_headers', array($this,'wpbe_wp_headers'));
            add_filter('user_contactmethods', array($this, 'wpbe_user_contactmethods'), 999, 1);
        }

        /** 
         * Alter information directly on plugins loaded
         *
         *  @since 5.2.0 
         */
        public function wpbe_plugins_loaded(){
            add_filter('plugin_action_links_'.plugin_basename(__FILE__), array($this, 'wpbe_plugin_action_links'));
        }

        /**
		 * Adds plugin action links
         * 
         * @since 5.2.0
		 */
		public function wpbe_plugin_action_links($links){
			$plugin_links = array(
                '<a href="/wp-admin/options-general.php?page='.$this->settings_page_endpoint.'">'.__('Settings', 'wp-basic-elements').'</a>',
                '<a href="'.$this->donate_link_paypal.'" target="_blank">'.__('Donate (Paypal)','wp-basic-elements').'</a>',
                '<a href="'.$this->donate_link_patreon.'" target="_blank">'.__('Donate (Patreon)','wp-basic-elements').'</a>',
			);

			return array_merge($plugin_links, $links);
		}

        /**
         * Initiate the WPBE Plugin Settings
         * 
         * @since 5.2.10
         */
        public function init_wpbe_settings(){
            // Load languages
            load_plugin_textdomain('wp-basic-elements', false, basename(dirname(__FILE__)).'/languages');

            $this->wpcore = array(
                'enable' => array(
                    'wpbe_enable_img_edit_overwrite' => __('Enable removal of saved edits of images to save space on server','wp-basic-elements'),
                    'wpbe_enable_shortcode_excerpts' => __('Enable shortcode in manual excerpts','wp-basic-elements'),
                    'wpbe_enable_side_menu_link'     => __('Enable shortcut for WP Basic Elements in sidebar menu','wp-basic-elements'),
                ),
                'disable' => array(
                    'wpbe_disable_wp_cron'              => __('Disable WP Cron','wp-basic-elements'),
                    'wpbe_disable_wp_rest_api'          => __('Disable WP Rest API for non loggedin users','wp-basic-elements'),
                    'wpbe_disable_wp_embed_scripts'     => __('Disable WP Embed scripts','wp-basic-elements'),
                    'wpbe_disable_wp_auto_updates'      => __('Disable automatic updates of WordPress, Themes and Plugins','wp-basic-elements'),
                    'wpbe_disable_themes_auto_updates'  => __('Disable automatic updates of Themes','wp-basic-elements'),
                    'wpbe_disable_plugins_auto_updates' => __('Disable automatic updates of Plugins','wp-basic-elements'),
                    'wpbe_disable_commenting'           => __('Disable the comments system','wp-basic-elements'),
                    'wpbe_disable_wp_file_edit'         => __('Disable plugin and theme file edit permission in admin for users','wp-basic-elements'),
                    'wpbe_disable_wp_notifications'     => __('Disable plugin, theme and WordPress update notifications for non-admins','wp-basic-elements'),
                ),
            );

            $this->wphead = array(
                'enable' => array(),
                'disable' => array(
                    'wpbe_disable_rest'      => __('Disable WP Rest API links','wp-basic-elements'),
                    'wpbe_disable_oembed'    => __('Disable WP Oembed discovery links','wp-basic-elements'),
                    'wpbe_disable_short'     => __('Disable WP shortlink','wp-basic-elements'),
                    'wpbe_disable_pings'     => __('Disable WP Pingbacks / Trackbacks','wp-basic-elements'),
                    'wpbe_disable_emojis'    => __('Disable Emoji icons if not in use.','wp-basic-elements'),
                    'wpbe_disable_rss'       => __('Disable Post, Comment and Category feeds','wp-basic-elements'),
                    'wpbe_disable_rsd'       => __('Disable EditURI link','wp-basic-elements'),
                    'wpbe_disable_manifest'  => __('Disable Windows Live Writer Manifest File','wp-basic-elements'),
                    'wpbe_disable_index'     => __('Disable Index link','wp-basic-elements'),
                    'wpbe_disable_adjacent'  => __('Disable Relational links for the Posts','wp-basic-elements'),
                    'wpbe_disable_block_css' => __('Disable Block styling','wp-basic-elements'),
                    'wpbe_disable_generator' => __('Disable meta generator tags','wp-basic-elements'),
                    'wpbe_disable_canonical' => __('Disable Canonical link','wp-basic-elements'),
                ),
            );

            $this->wpdashboard = array(
                'enable' => array(),
                'disable' => array(
                    'wpbe_disable_welcome_panel'      => __('Disable Welcome to WordPress','wp-basic-elements'),
                    'wpbe_disable_dashboard_glance'   => __('Disable At a Glance','wp-basic-elements'),
                    'wpbe_disable_dashboard_activity' => __('Disable Activity','wp-basic-elements'),
                    'wpbe_disable_dashboard_press'    => __('Disable Quick Draft','wp-basic-elements'),
                    'wpbe_disable_dashboard_wpnews'   => __('Disable WordPress Events and News','wp-basic-elements'),
                ),
            );

            $this->wpadmintoolbar = array(
                'enable' => array(),
                'disable' => array(
                    'wpbe_disable_wp_logo'        => __('Disable WP Logo','wp-basic-elements'),
                    'wpbe_disable_wp_customize'   => __('Disable WP Customize','wp-basic-elements'),
                    'wpbe_disable_wp_updates'     => __('Disable WP Updates','wp-basic-elements'),
                    'wpbe_disable_wp_comments'    => __('Disable WP Comments','wp-basic-elements'),
                    'wpbe_disable_wp_search'      => __('Disable WP Search','wp-basic-elements'),
                    'wpbe_disable_wp_sitename'    => __('Disable WP Sitename','wp-basic-elements'),
                    'wpbe_disable_wp_edit'        => __('Disable WP Edit','wp-basic-elements'),
                    'wpbe_disable_wp_new_content' => __('Disable WP New Content','wp-basic-elements'),
                ),
            );

            $this->wpusers = array(
                'enable' => array(),
                'disable' => array(
                    'wpbe_disable_contact_lang'       => __('Disable language chooser for users','wp-basic-elements'),
                    'wpbe_disable_contact_colors'     => __('Disable Color Scheme selector for users','wp-basic-elements'),
                    'wpbe_disable_contact_facebook'   => __('Disable Facebook field from users contact field','wp-basic-elements'),
                    'wpbe_disable_instagram_contact'  => __('Disable Instagram field from users contact field','wp-basic-elements'),
                    'wpbe_disable_linkedin_contact'   => __('Disable LinkedIn field from users contact field','wp-basic-elements'),
                    'wpbe_disable_myspace_contact'    => __('Disable MySpace field from users contact field','wp-basic-elements'),
                    'wpbe_disable_pinterest_contact'  => __('Disable Pinterest field from users contact field','wp-basic-elements'),
                    'wpbe_disable_soundcloud_contact' => __('Disable SoundCloud field from users contact field','wp-basic-elements'),
                    'wpbe_disable_tumblr_contact'     => __('Disable Tumblr field from users contact field','wp-basic-elements'),
                    'wpbe_disable_twitter_contact'    => __('Disable Twitter field from users contact field','wp-basic-elements'),
                    'wpbe_disable_youtube_contact'    => __('Disable YouTube field from users contact field','wp-basic-elements'),
                    'wpbe_disable_wikipedia_contact'  => __('Disable WikiPedia field from users contact field','wp-basic-elements'),
                    'wpbe_disable_contact_website'    => __('Disable Website field from users contact field','wp-basic-elements'),
                ),
            );

            $this->wpadminfooter = array(
                'wpbe_admin_footer_left'  => __('Change the default left footer text to what you want.','wp-basic-elements'),
                'wpbe_admin_footer_right' => __('Change the default right footer text to what you want.','wp-basic-elements'),
            );

            $this->gutenberg = array(
                'enable' => array(),
                'disable' => array(
                    'wpbe_disable_gtb_fullscreen'   => __('Disable Fullscreen Mode','wp-basic-elements'),
                    'wpbe_disable_gtb_welcome' => __('Disable Welcome Guide','wp-basic-elements'),
                ),
            );

            $this->plugins = array(
                'enable' => array(),
                'disable' => array(
                    'wpbe_disable_yseo_menu'         => __('Disable Yoast SEO in toolbar menu','wp-basic-elements'),
                    'wpbe_disable_yseo_dashboard'    => __('Disable Yoast SEO Posts Overview','wp-basic-elements'),
                    'wpbe_disable_wc_reviews'        => __('Disable WooCommerce recent reviews','wp-basic-elements'),
                    'wpbe_disable_wc_status'         => __('Disable WooCommerce status','wp-basic-elements'),
                    'wpbe_disable_bbpress_right_now' => __('Disable bbPress right now','wp-basic-elements'),
                ),
            );

            $this->wpmail = array(
                'wpbe_wp_mail_from_name'   => __('Change mail name <strong>(WordPress)</strong> sent to users to your own','wp-basic-elements'),
                'wpbe_wp_mail_from_adress' => __('Change mail adress <strong>(wordpress@mysite.com)</strong> sent to users','wp-basic-elements'),
            );

            // Get settings from the database
            $wpbe_options_settings = get_option('wpbe_settings');
            $this->wpbe_settings = ($wpbe_options_settings !== '') ? unserialize(get_option('wpbe_settings')) : '';
            
            // Set correct output of the settings
            $this->wpbe_keys = is_array($this->wpbe_settings) ? $this->wpbe_settings : [];
        }

        /**
         * Check that the key exists and returns valid result 
         * 
         * @since 5.2.0
         */
        public function wpbe_valid_key($key){
            return ($this->wpbe_keys && (array_key_exists($key, $this->wpbe_keys) && $this->wpbe_keys[$key] == 'true')) ? true : false;
        }

        /**
         * Check that the data exists and returns valid results 
         * 
         * @since 5.2.5
         */
        public function wpbe_valid_data($key){
            return ($this->wpbe_keys && (array_key_exists($key, $this->wpbe_keys) && $this->wpbe_keys[$key] !== '')) ? true : false;
        }

        /**
         * Alter the wp admin menues in sidebar 
         * 
         * @since 5.2.3
         */
        public function wpbe_admin_enqueue_scripts(){
            wp_enqueue_style('wp-basic-elements-style', plugin_dir_url(__FILE__).'assets/css/wpbe-styles.css', false, WBE_VERSION, false);
            wp_enqueue_script('wp-basic-elements-script', plugin_dir_url(__FILE__).'assets/js/wpbe-scripts.js', false, WBE_VERSION, true);
        
            // Remove Language chooser from User profile
            if($this->wpbe_valid_key('wpbe_disable_contact_lang')){ ?>
                <script>document.addEventListener("DOMContentLoaded", function() { const wpbeLanguageWrap = document.querySelector('tr.user-language-wrap'); if(wpbeLanguageWrap){ wpbeLanguageWrap.parentNode.removeChild(wpbeLanguageWrap); }});</script>
            <?php }     
        }

        /**
         * Alter the wp admin menues in sidebar 
         * 
         * @since 5.2.0
         */
        public function wpbe_admin_menu(){
            add_submenu_page('options-general.php', __('WP Basic Elements','wp-basic-elements'), __('WP Basic Elements','wp-basic-elements'), 'manage_options', $this->settings_page_endpoint, array($this, $this->settings_page_endpoint));
            
            if($this->wpbe_valid_key('wpbe_enable_side_menu_link')){ 
                add_menu_page(__('WPB Elements','wp-basic-elements'), __('WPB Elements','wp-basic-elements'), 'manage_options', __FILE__, $this->settings_page_endpoint, 'dashicons-code-standards', 3); 
            }

            if($this->wpbe_valid_key('wpbe_disable_commenting')){ 
                remove_menu_page('edit-comments.php'); 
            }
        }

        /**
         * Create and display the Settings Page in admin
         * 
         * @since 5.2.0
         */
        public function wpbe_settings_page(){ ?>
            <form id="wpbe_elements_form">
                <div class="wpbe-header">
                    <h1><?php _e('WP Basic Elements','wp-basic-elements'); ?></h1>
                    <nav class="wpbe-tabs-wrapper">
                        <a href="#" class="wpbe-tab active" data-id="wpcore"><?php echo __('Core','wp-basic-elements').' <small>('.count($this->wpcore['enable'] + $this->wpcore['disable']).')</small>'; ?></a>
                        <a href="#" class="wpbe-tab" data-id="wphead"><?php echo __('Head','wp-basic-elements').' <small>('.count($this->wphead['enable'] + $this->wphead['disable']).')</small>'; ?></a>
                        <a href="#" class="wpbe-tab" data-id="wpdashboard"><?php echo __('Dashboard','wp-basic-elements').' <small>('.count($this->wpdashboard['enable'] + $this->wpdashboard['disable']).')</small>'; ?></a>
                        <a href="#" class="wpbe-tab" data-id="wpadmintoolbar"><?php echo __('Admin toolbar','wp-basic-elements').' <small>('.count($this->wpadmintoolbar['enable'] + $this->wpadmintoolbar['disable']).')</small>'; ?></a>
                        <a href="#" class="wpbe-tab" data-id="wpadminfooter"><?php echo __('Admin footer','wp-basic-elements').' <small>('.count($this->wpadminfooter).')</small>'; ?></a>
                        <a href="#" class="wpbe-tab" data-id="wpusers"><?php echo __('User profile','wp-basic-elements').' <small>('.count($this->wpusers['enable'] + $this->wpusers['disable']).')</small>'; ?></a>
                        <a href="#" class="wpbe-tab" data-id="gutenberg"><?php echo __('Gutenberg','wp-basic-elements').' <small>('.count($this->gutenberg['enable'] + $this->gutenberg['disable']).')</small>'; ?></a>
                        <a href="#" class="wpbe-tab" data-id="plugins"><?php echo __('Plugins','wp-basic-elements').' <small>('.count($this->plugins['enable'] + $this->plugins['disable']).')</small>'; ?></a>
                        <a href="#" class="wpbe-tab" data-id="wpmail"><?php echo __('Mail','wp-basic-elements').' <small>('.count($this->wpmail).')</small>'; ?></a>
                    </nav>
                </div>
                <div class="wpbe-check-body">
                    <div id="wpcore" class="div-tab">
                    <?php foreach($this->wpcore as $uni => $core){ ?>
                        <?php if($core){ ?>
                            <div class="notice notice-<?php echo ($uni == 'enable') ? 'info' : 'error'; ?> inline">
                                <?php 
                                    if($uni == 'enable' && $core) { echo '<p>'.sprintf(__('Enable desired &laquo; %s &raquo; functions.','wp-basic-elements'), __('Core','wp-basic-elements')).'</p>'; }
                                    elseif($uni == 'disable' && $core){ echo '<p>'.sprintf(__('Disable undesired &laquo; %s &raquo; functions.','wp-basic-elements'), __('Core','wp-basic-elements')).'</p>'; }
                                ?>
                            </div>
                        <?php } ?>
                        <?php
                            foreach($core as $key => $desc){ ?>
                                <h3 class="<?php echo $uni; ?>">
                                    <span><?php echo $desc; ?></span>
                                    <label class="switch">
                                        <input id="<?php echo $key; ?>" type="checkbox" name="<?php echo $key; ?>" value="1" <?php echo (($this->wpbe_valid_key($key)) ? 'checked="checked"' : ''); ?>>
                                        <span for="<?php echo $key; ?>" class="slider"></span>
                                    </label>
                                </h3>
                        <?php }
                        }
                    ?>
                    </div>
                    <div id="wphead" class="div-tab">
                    <?php foreach($this->wphead as $uni => $core){ ?>
                        <?php if($core){ ?>
                            <div class="notice notice-<?php echo ($uni == 'enable') ? 'info' : 'error'; ?> inline">
                                <?php 
                                    if($uni == 'enable' && $core) { echo '<p>'.sprintf(__('Enable desired &laquo; %s &raquo; functions.','wp-basic-elements'), __('Head','wp-basic-elements')).'</p>'; }
                                    elseif($uni == 'disable' && $core){ echo '<p>'.sprintf(__('Disable undesired &laquo; %s &raquo; functions.','wp-basic-elements'), __('Head','wp-basic-elements')).'</p>'; }
                                ?>
                            </div>
                        <?php } ?>
                        <?php
                            foreach($core as $key => $desc){ ?>
                                <h3 class="<?php echo $uni; ?>">
                                    <span><?php echo $desc; ?></span>
                                    <label class="switch">
                                        <input id="<?php echo $key; ?>" type="checkbox" name="<?php echo $key; ?>" value="1" <?php echo (($this->wpbe_valid_key($key)) ? 'checked="checked"' : ''); ?>>
                                        <span for="<?php echo $key; ?>" class="slider"></span>
                                    </label>
                                </h3>
                        <?php }
                        }
                    ?>
                    </div>
                    <div id="wpdashboard" class="div-tab">
                    <?php foreach($this->wpdashboard as $uni => $core){ ?>
                        <?php if($core){ ?>
                            <div class="notice notice-<?php echo ($uni == 'enable') ? 'info' : 'error'; ?> inline">
                                <?php 
                                    if($uni == 'enable' && $core) { echo '<p>'.sprintf(__('Enable desired &laquo; %s &raquo; functions.','wp-basic-elements'), __('Dashboard','wp-basic-elements')).'</p>'; }
                                    elseif($uni == 'disable' && $core){ echo '<p>'.sprintf(__('Disable undesired &laquo; %s &raquo; functions.','wp-basic-elements'), __('Dashboard','wp-basic-elements')).'</p>'; }
                                ?>
                            </div>
                        <?php } ?>
                        <?php
                            foreach($core as $key => $desc){ ?>
                                <h3 class="<?php echo $uni; ?>">
                                    <span><?php echo $desc; ?></span>
                                    <label class="switch">
                                        <input id="<?php echo $key; ?>" type="checkbox" name="<?php echo $key; ?>" value="1" <?php echo (($this->wpbe_valid_key($key)) ? 'checked="checked"' : ''); ?>>
                                        <span for="<?php echo $key; ?>" class="slider"></span>
                                    </label>
                                </h3>
                        <?php }
                        }
                    ?>
                    </div>
                    <div id="wpadmintoolbar" class="div-tab">
                    <?php foreach($this->wpadmintoolbar as $uni => $core){ ?>
                        <?php if($core){ ?>
                            <div class="notice notice-<?php echo ($uni == 'enable') ? 'info' : 'error'; ?> inline">
                                <?php 
                                    if($uni == 'enable' && $core) { echo '<p>'.sprintf(__('Enable desired &laquo; %s &raquo; functions.','wp-basic-elements'), __('Admin toolbar','wp-basic-elements')).'</p>'; }
                                    elseif($uni == 'disable' && $core){ echo '<p>'.sprintf(__('Disable undesired &laquo; %s &raquo; functions.','wp-basic-elements'), __('Admin toolbar','wp-basic-elements')).'</p>'; }
                                ?>
                            </div>
                        <?php } ?>
                        <?php
                            foreach($core as $key => $desc){ ?>
                                <h3 class="<?php echo $uni; ?>">
                                    <span><?php echo $desc; ?></span>
                                    <label class="switch">
                                        <input id="<?php echo $key; ?>" type="checkbox" name="<?php echo $key; ?>" value="1" <?php echo (($this->wpbe_valid_key($key)) ? 'checked="checked"' : ''); ?>>
                                        <span for="<?php echo $key; ?>" class="slider"></span>
                                    </label>
                                </h3>
                        <?php }
                        }
                    ?>
                    </div>
                    <div id="wpadminfooter" class="div-tab">
                        <?php foreach($this->wpadminfooter as $key => $desc){ ?>
                            <h3>
                                <span><?php echo $desc; ?></span>
                            </h3>
                            <?php wp_editor((($this->wpbe_valid_data($key)) ? $this->wpbe_keys[$key] : ''), $key, $this->wp_editor_settings); ?>
                        <?php } ?>
                    </div>
                    <div id="wpusers" class="div-tab">
                    <?php foreach($this->wpusers as $uni => $core){ ?>
                        <?php if($core){ ?>
                            <div class="notice notice-<?php echo ($uni == 'enable') ? 'info' : 'error'; ?> inline">
                                <?php 
                                    if($uni == 'enable' && $core) { echo '<p>'.sprintf(__('Enable desired &laquo; %s &raquo; functions.','wp-basic-elements'), __('User profile','wp-basic-elements')).'</p>'; }
                                    elseif($uni == 'disable' && $core){ echo '<p>'.sprintf(__('Disable undesired &laquo; %s &raquo; functions.','wp-basic-elements'), __('User profile','wp-basic-elements')).'</p>'; }
                                ?>
                            </div>
                        <?php } ?>
                        <?php
                            foreach($core as $key => $desc){ ?>
                                <h3 class="<?php echo $uni; ?>">
                                    <span><?php echo $desc; ?></span>
                                    <label class="switch">
                                        <input id="<?php echo $key; ?>" type="checkbox" name="<?php echo $key; ?>" value="1" <?php echo (($this->wpbe_valid_key($key)) ? 'checked="checked"' : ''); ?>>
                                        <span for="<?php echo $key; ?>" class="slider"></span>
                                    </label>
                                </h3>
                        <?php }
                        }
                    ?>
                    </div>
                    <div id="gutenberg" class="div-tab">
                    <?php foreach($this->gutenberg as $uni => $core){ ?>
                        <?php if($core){ ?>
                            <div class="notice notice-<?php echo ($uni == 'enable') ? 'info' : 'error'; ?> inline">
                                <?php 
                                    if($uni == 'enable' && $core) { echo '<p>'.sprintf(__('Enable desired &laquo; %s &raquo; functions.','wp-basic-elements'), __('Gutenberg','wp-basic-elements')).'</p>'; }
                                    elseif($uni == 'disable' && $core){ echo '<p>'.sprintf(__('Disable undesired &laquo; %s &raquo; functions.','wp-basic-elements'), __('Gutenberg','wp-basic-elements')).'</p>'; }
                                ?>
                            </div>
                        <?php } ?>
                        <?php
                            foreach($core as $key => $desc){ ?>
                                <h3 class="<?php echo $uni; ?>">
                                    <span><?php echo $desc; ?></span>
                                    <label class="switch">
                                        <input id="<?php echo $key; ?>" type="checkbox" name="<?php echo $key; ?>" value="1" <?php echo (($this->wpbe_valid_key($key)) ? 'checked="checked"' : ''); ?>>
                                        <span for="<?php echo $key; ?>" class="slider"></span>
                                    </label>
                                </h3>
                        <?php }
                        }
                    ?>
                    </div>
                    <div id="plugins" class="div-tab">
                    <?php foreach($this->plugins as $uni => $core){ ?>
                        <?php if($core){ ?>
                            <div class="notice notice-<?php echo ($uni == 'enable') ? 'info' : 'error'; ?> inline">
                                <?php 
                                    if($uni == 'enable' && $core) { echo '<p>'.sprintf(__('Enable desired &laquo; %s &raquo; functions.','wp-basic-elements'), __('Plugins','wp-basic-elements')).'</p>'; }
                                    elseif($uni == 'disable' && $core){ echo '<p>'.sprintf(__('Disable undesired &laquo; %s &raquo; functions.','wp-basic-elements'), __('Plugins','wp-basic-elements')).'</p>'; }
                                ?>
                            </div>
                        <?php } ?>
                        <?php
                            foreach($core as $key => $desc){ ?>
                                <h3 class="<?php echo $uni; ?>">
                                    <span><?php echo $desc; ?></span>
                                    <label class="switch">
                                        <input id="<?php echo $key; ?>" type="checkbox" name="<?php echo $key; ?>" value="1" <?php echo (($this->wpbe_valid_key($key)) ? 'checked="checked"' : ''); ?>>
                                        <span for="<?php echo $key; ?>" class="slider"></span>
                                    </label>
                                </h3>
                        <?php }
                        }
                    ?>
                    </div>
                    <div id="wpmail" class="div-tab">
                        <?php foreach($this->wpmail as $key => $desc){ ?>
                            <h3>
                                <span><?php echo $desc; ?></span>
                            </h3>
                            <input type="text" name="<?php echo $key; ?>" value="<?php echo ($this->wpbe_valid_data($key)) ? $this->wpbe_keys[$key] : ''; ?>" />
                        <?php } ?>
                    </div>
                    <div id="wpbe_popup">
                        <div id="wpbe_popup_container">
                            <span id="wpbe_icon" class="dashicons dashicons-yes"></span>
                            <h2 id="wpbe_title"><?php _e('Saved','wp-basic-elements'); ?></h2>
                            <p id="wpbe_content"><?php _e('All your settings have been saved.','wp-basic-elements'); ?></p>
                            <button id="wpbe_button" type="button"><?php _e('Ok','wp-basic-elements'); ?></button>
                        </div>
                    </div>
                    <p class="wpbe-footer">
                        <input type="submit" name="submit" id="submit" class="button button-primary" value="<?php _e('Save Changes','wp-basic-elements'); ?>">
                        <a href="https://twitter.com/damircalusic/" target="_blank"><?php _e('Follow on Twitter','wp-basic-elements'); ?></a> | 
                        <a href="<?php echo $this->donate_link_paypal; ?>" target="_blank"><?php _e('Donate (Paypal)','wp-basic-elements'); ?></a> | 
                        <a href="<?php echo $this->donate_link_patreon; ?>" target="_blank"><?php _e('Donate (Patreon)','wp-basic-elements'); ?></a>
                        <span id="info" class="dashicons dashicons-info"></span>
                        <span id="version"><?php echo WBE_VERSION; ?></span>
                    </p>
                    <div id="wpbe-info-notice" class="notice notice-info inline"><p><?php _e('With WP Basic Elements you can disable unnecessary features and speed up your site. Make the WP Admin simple and clean. You can change admin footers in backend, activate shortcodes in widgets, remove admin toolbar options and you can clean the code markup from unnecessary code snippets like WordPress generator meta tag and a bunch of other non important code snippets in the code. Cleaning the code markup will speed up your sites loadtime and increase the overall performance.','wp-basic-elements'); ?></p></div>
                </div>
            </form>
        <?php 
        }

        /** 
         * Save settings for the plugin
         * 
         * @since 5.2.3
         */
        public function wpbe_save_settings(){
            $data = ($_POST['data']) ? $_POST['data'] : '';
            
            if($data){
                $cleanedSettings = $this->wpbe_clean_settings($data);
                $settings = serialize($cleanedSettings);
                update_option('wpbe_settings', $settings);
            }
            
            die();
        }

        /** 
         * Clean settings from unwanted special characters
         * 
         * @since 5.2.3
         */
        public function wpbe_clean_settings($data){
            if(!is_array($data)) {
                return stripslashes($data);
            }
        
            $newSettings = [];
            
            foreach($data as $key => $value){
                $newSettings[$key] = $this->wpbe_clean_settings($value);
            }

            return $newSettings;
        }

        /**
         * Alter functions and data on init
         * 
         * @since 5.2.6
         */
        public function wpbe_init(){
            global $pagenow;

            // Run the needed settings initiatings
            $this->init_wpbe_settings();
            
            // Enable so edits of images are removed to save space on server
            if($this->wpbe_valid_key('wpbe_enable_img_edit_overwrite')){
                if(!defined('IMAGE_EDIT_OVERWRITE')){
                    define('IMAGE_EDIT_OVERWRITE', true);
                }
            }

            // Disable WP Cron
            if($this->wpbe_valid_key('wpbe_disable_wp_cron')){
                if(!defined('DISABLE_WP_CRON')){
                    define('DISABLE_WP_CRON', true);
                }
            }

            // Disable File editing in admin
            if($this->wpbe_valid_key('wpbe_disable_wp_file_edit')){
                if(!defined('DISALLOW_FILE_EDIT')){
                    define('DISALLOW_FILE_EDIT', true);
                }
            }

            // Disable automatic updating of WordPress, Themes and Plugins
            if($this->wpbe_valid_key('wpbe_disable_wp_auto_updates')){
                if(!defined('AUTOMATIC_UPDATER_DISABLED')){
                    define('AUTOMATIC_UPDATER_DISABLED', true);
                }
            }
    
            // Prevent users from accessing comments page in admin and remove support for all post types
            if($this->wpbe_valid_key('wpbe_disable_commenting')){
                if($pagenow === 'edit-comments.php'){
                    wp_redirect(admin_url());
                    exit;
                }

                foreach(get_post_types() as $post_type){
                    if(post_type_supports($post_type, 'comments')){
                        remove_post_type_support($post_type, 'comments');
                    }
                }
            }

            // Disable all support of trackbacks/pings on all post types
            if($this->wpbe_valid_key('wpbe_disable_pings')){
                foreach(get_post_types() as $post_type){
                    if(post_type_supports($post_type, 'trackbacks')){
                        remove_post_type_support($post_type, 'trackbacks');
                    }
                }
            }

            // Remove Actions
            if($this->wpbe_valid_key('wpbe_disable_welcome_panel')){ remove_action('welcome_panel', 'wp_welcome_panel'); }      // Welcome to WordPress Dashboard
            if($this->wpbe_valid_key('wpbe_disable_contact_colors')){ remove_action('admin_color_scheme_picker', 'admin_color_scheme_picker'); }  // Admin color chooser from Users contact info
            if($this->wpbe_valid_key('wpbe_disable_index')){ remove_action('wp_head', 'index_rel_link'); }                      // Index link
            if($this->wpbe_valid_key('wpbe_disable_adjacent')){ remove_action('wp_head', 'adjacent_posts_rel_link_wp_head'); }  // Relational links (Prev / Start)
            if($this->wpbe_valid_key('wpbe_disable_short')){ remove_action('wp_head', 'wp_shortlink_wp_head', 10); }            // WordPress shortlink
            if($this->wpbe_valid_key('wpbe_disable_rsd')){ remove_action('wp_head', 'rsd_link'); }                              // EditURI link (Really Simple Discovery service endpoint)
            if($this->wpbe_valid_key('wpbe_disable_manifest')){ remove_action('wp_head', 'wlwmanifest_link'); }                 // Windows Live Writer manifest file
            if($this->wpbe_valid_key('wpbe_disable_canonical')){ remove_action('wp_head', 'rel_canonical'); }                   // Canonical link
            if($this->wpbe_valid_key('wpbe_disable_oembed')){ remove_action('wp_head', 'wp_oembed_add_discovery_links', 10); }  // WP Oembed
            if($this->wpbe_valid_key('wpbe_disable_generator')){                                                                // Generator meta tags
                remove_action('wp_head', 'wp_generator');                                                                       // - WordPress / WooCommerce
                remove_action('wp_head', 'mnthemes_add_meta_information_action', 99);                                           // - Product Filters for WooCommerce

                // Check for WPML
                if(function_exists('icl_object_id')){                                                                           // - WPML Multilingual
                    global $sitepress;
                    remove_action('wp_head', array($sitepress, 'meta_generator_tag')); 
                }
            }
            if($this->wpbe_valid_key('wpbe_disable_rss') == '1'){                                                               // Feeds
                remove_action('wp_head', 'feed_links', 2);                                                                      // - Post and Comments feeds
                remove_action('wp_head', 'feed_links_extra', 3);                                                                // - Category feeds
            }
            if($this->wpbe_valid_key('wpbe_disable_rest') == '1'){                                                              // REST links
                remove_action('wp_head', 'rest_output_link_wp_head', 10);
                remove_action('template_redirect', 'rest_output_link_header', 11); 
                remove_action('xmlrpc_rsd_apis', 'rest_output_rsd');
            }
            if($this->wpbe_valid_key('wpbe_disable_emojis')){                                                                   // Disable Emoji icons
                remove_action('wp_head', 'print_emoji_detection_script', 7);
                remove_action('admin_print_scripts', 'print_emoji_detection_script');
                remove_action('wp_print_styles', 'print_emoji_styles');
                remove_action('admin_print_styles', 'print_emoji_styles');	
                remove_filter('the_content_feed', 'wp_staticize_emoji');
                remove_filter('comment_text_rss', 'wp_staticize_emoji');	
                remove_filter('wp_mail', 'wp_staticize_emoji_for_email');
                add_filter('tiny_mce_plugins', array($this, 'wpbe_tiny_mce_plugins'));
                add_filter('emoji_svg_url', '__return_false');  
            }

            // Add Filters
            if($this->wpbe_valid_key('wpbe_disable_themes_auto_updates')){ add_filter('themes_auto_update_enabled', '__return_false'); }            // Disable automatic update of Themes 
            if($this->wpbe_valid_key('wpbe_disable_plugins_auto_updates')){ add_filter('plugins_auto_update_enabled', '__return_false'); }          // Disable automatic update of Plugins   
            if($this->wpbe_valid_data('wpbe_admin_footer_left')){ add_filter('admin_footer_text', array($this, 'wpbe_admin_footer_left')); }        // Add custom text to the admin footer left
            if($this->wpbe_valid_data('wpbe_admin_footer_right')){ add_filter('update_footer', array($this, 'wpbe_admin_footer_right'), 11); }      // Add custom text to the admin footer right
            if($this->wpbe_valid_data('wpbe_wp_mail_from_name')){ add_filter('wp_mail_from_name', array($this, 'wpbe_wp_mail_from_name')); }        // Change WordPress custom name to your own name
            if($this->wpbe_valid_data('wpbe_wp_mail_from_adress')){ add_filter('wp_mail_from', array($this, 'wpbe_wp_mail_from_adress')); }         // Change WordPress custom name
            if($this->wpbe_valid_key('wpbe_enable_shortcode_excerpts')){ add_filter('get_the_excerpt', 'do_shortcode'); }                           // Shortcode ability for Excerpts
            if($this->wpbe_valid_key('wpbe_disable_pings')){                                                                                        // WP Pings
                add_action('pre_ping', array($this, 'wpbe_pre_ping'));                                                                              // - Disable self pinging
                add_filter('pings_open', '__return_false', 20, 2);                                                                                  // - Disable pinging on frontend
            }
            if($this->wpbe_valid_key('wpbe_disable_commenting')){                                                                                   // WP Commenting
                add_action('widgets_init', array($this, 'wpbe_widgets_init'));                                                                      // - Remove comments stylkesheet from WP Head
                add_filter('comments_open', '__return_false', 20, 2);                                                                               // - Close comments on the front-end
                add_filter('comments_array', '__return_empty_array', 10, 2);                                                                        // - Hide existing comments
            }
            if($this->wpbe_valid_key('wpbe_disable_notifications')){                                                                                // Disable system notifications for non-admins
                add_filter('pre_site_transient_update_core', array($this, 'wpbe_disable_core_updates'));
                add_filter('pre_site_transient_update_plugins', array($this, 'wpbe_disable_core_updates'));
                add_filter('pre_site_transient_update_themes', array($this, 'wpbe_disable_core_updates'));
            }
            if($this->wpbe_valid_key('wpbe_disable_wp_rest_api')){
                add_filter('rest_authentication_errors',  array($this, 'wpbe_disable_wp_rest_api'));
            }
        }

        /**
         * Filter function used to remove/add elements to the tinymce editor.
         * 
         * @since 5.2.0
         */
        function wpbe_tiny_mce_plugins($plugins){
            if(is_array($plugins)){
                return array_diff($plugins, array('wpemoji'));
            }
            else{
                return array();
            }
        }

        /**
         * Disable Plugins, WordPress and Themes update notifications for non-admins
         * 
         * @since 5.2.0
         */
        public function wpbe_disable_core_updates(){
            if(!current_user_can('administrator')){
                global $wp_version;
                return (object) array('last_checked'=> time(), 'version_checked'=> $wp_version, 'updates' => array());
            }
        }

        /**
         * Disable WP Rest API for non loggedin users
         * 
         * @since 5.2.6
         */
        public function wpbe_disable_wp_rest_api($access){
            if(!is_user_logged_in()){
                $message = apply_filters('disable_wp_rest_api_error', __('REST API restricted to authenticated users.', 'wp-basic-elements'));
                return new WP_Error('rest_login_required', $message, array('status' => rest_authorization_required_code()));  
            }
            
            return $access;
        }

        /**
         * Alter the WP Admin menues in toolbar
         * 
         * @since 5.2.0
         */
        public function wpbe_wp_before_admin_bar_render(){
            global $wp_admin_bar; 
            
            if($this->wpbe_valid_key('wpbe_disable_wp_logo')){ $wp_admin_bar->remove_menu('wp-logo'); }              // WP Logo
            if($this->wpbe_valid_key('wpbe_disable_wp_new_content')){ $wp_admin_bar->remove_menu('new-content'); }   // WP New Content
            if($this->wpbe_valid_key('wpbe_disable_wp_sitename')){ $wp_admin_bar->remove_menu('site-name'); }        // WP Site Name
            if($this->wpbe_valid_key('wpbe_disable_wp_customize')){ $wp_admin_bar->remove_menu('customize'); }       // WP Customize
            if($this->wpbe_valid_key('wpbe_disable_wp_edit')){ $wp_admin_bar->remove_menu('edit'); }                 // WP Edit
            if($this->wpbe_valid_key('wpbe_disable_wp_updates')){ $wp_admin_bar->remove_menu('updates'); }           // WP Updates
            if($this->wpbe_valid_key('wpbe_disable_wp_search')){ $wp_admin_bar->remove_menu('search'); }             // WP Search
            if($this->wpbe_valid_key('wpbe_disable_wp_comments')){ $wp_admin_bar->remove_menu('comments'); }         // WP Comments
            if($this->wpbe_valid_key('wpbe_disable_yseo_menu')){ $wp_admin_bar->remove_menu('wpseo-menu'); }         // Yoast SEO
        }

        /**
         * Alter and modify the admin dashboard
         * 
         * @since 5.2.0
         */
        public function wpbe_wp_dashboard_setup(){
            if($this->wpbe_valid_key('wpbe_disable_dashboard_glance')){ remove_meta_box('dashboard_right_now', 'dashboard', 'normal'); }              // At a Glance
            if($this->wpbe_valid_key('wpbe_disable_dashboard_activity')){ remove_meta_box('dashboard_activity', 'dashboard', 'normal'); }             // Activity
            if($this->wpbe_valid_key('wpbe_disable_dashboard_press')){ remove_meta_box('dashboard_quick_press', 'dashboard', 'side'); }               // Quick Press
            if($this->wpbe_valid_key('wpbe_disable_dashboard_wpnews')){ remove_meta_box('dashboard_primary', 'dashboard', 'side'); }                  // WordPress News
            if($this->wpbe_valid_key('wpbe_disable_yseo_dashboard')){ remove_meta_box('wpseo-dashboard-overview', 'dashboard', 'normal'); }           // Yoast SEO
            if($this->wpbe_valid_key('wpbe_disable_wc_reviews')){ remove_meta_box('woocommerce_dashboard_recent_reviews', 'dashboard', 'normal'); }   // WooCommerce Reviews
            if($this->wpbe_valid_key('wpbe_disable_wc_status')){ remove_meta_box('woocommerce_dashboard_status', 'dashboard', 'normal'); }            // WooCommerce Status
            if($this->wpbe_valid_key('wpbe_disable_bbpress_right_now')){ remove_meta_box('bbp-dashboard-right-now', 'dashboard', 'normal'); }         // bbPress Right now
        }

        /**
         * Alter the info in server headers
         * 
         * @since 5.2.0
         */
        public function wpbe_wp_headers($headers){
            if($this->wpbe_valid_key('wpbe_disable_pings')){ 
                unset($headers['X-Pingback']);                  // Unset Pings headers
            }
            
            return $headers;
        }

        /** 
         * Disable self pingbacks for pages and posts
         * 
         * @since 5.2.4
         */
        public function wpbe_pre_ping(&$links){
            if($this->wpbe_valid_key('wpbe_disable_pings')){ 
                $home = get_option('home');
                
                foreach($links as $l => $link){
                    if(0 === strpos( $link, $home)){
                        unset($links[$l]);
                    }
                }
            }
        }

        /** 
         * Remove comments stylesheet from frontend
         * 
         * @since 5.2.9
         */
        public function wpbe_widgets_init(){
            global $wp_widget_factory;
	
            if(array_key_exists('WP_Widget_Recent_Comments', $wp_widget_factory->widgets)){
                remove_action('wp_head', array($wp_widget_factory->widgets['WP_Widget_Recent_Comments'], 'recent_comments_style'));
            }
        }

        /**
         * Unset user information in admin profile section 
         * 
         * @since 5.2.0
         */
        public function wpbe_user_contactmethods($contactmethods) {
            if($this->wpbe_valid_key('wpbe_disable_contact_website')){ unset($contactmethods['url']); }              // Website URL
            if($this->wpbe_valid_key('wpbe_disable_contact_facebook')){ unset($contactmethods['facebook']); }        // Facebook
            if($this->wpbe_valid_key('wpbe_disable_instagram_contact')){ unset($contactmethods['instagram']); }      // Instagram
            if($this->wpbe_valid_key('wpbe_disable_linkedin_contact')){ unset($contactmethods['linkedin']); }        // LinkedIn
            if($this->wpbe_valid_key('wpbe_disable_myspace_contact')){ unset($contactmethods['myspace']); }          // MySpace
            if($this->wpbe_valid_key('wpbe_disable_pinterest_contact')){ unset($contactmethods['pinterest']); }      // Pinterest
            if($this->wpbe_valid_key('wpbe_disable_soundcloud_contact')){ unset($contactmethods['soundcloud']); }    // SoundCloud
            if($this->wpbe_valid_key('wpbe_disable_tumblr_contact')){ unset($contactmethods['tumblr']); }            // Tumblr
            if($this->wpbe_valid_key('wpbe_disable_twitter_contact')){ unset($contactmethods['twitter']); }          // Twitter
            if($this->wpbe_valid_key('wpbe_disable_youtube_contact')){ unset($contactmethods['youtube']); }          // YouTube
            if($this->wpbe_valid_key('wpbe_disable_wikipedia_contact')){ unset($contactmethods['wikipedia']); }      // WikiPedia

            return $contactmethods;
        }  

        /** 
         * Alter scripts and styles in header
         * 
         * @since 5.2.0
         */
        public function wpbe_wp_enqueue_scripts(){
            if(!is_admin()){
                if($this->wpbe_valid_key('wpbe_disable_wp_embed_scripts')){
                    wp_deregister_script('wp-embed');           // WP Embed
                }

                if($this->wpbe_valid_key('wpbe_disable_block_css')){ 
                    wp_deregister_style('wp-block-library');    // WordPress
                    wp_dequeue_style('wp-block-library');       // WordPress
                    wp_dequeue_style('wp-block-library-theme'); // WordPress
                    wp_dequeue_style('wc-block-style');         // WooCommerce 
                }
            }
        }

        /**
         * Modify Gutenberg Editor by custom scripts and styles
         * 
         * @since 5.2.8
         */
        function wpbe_enqueue_block_editor_assets(){
            $scripts = '';

            // Disable Fullscreen Mode
            if($this->wpbe_valid_key('wpbe_disable_gtb_fullscreen')){
                $scripts .= "jQuery(window).load(function(){ const isFullscreenMode = wp.data.select('core/edit-post').isFeatureActive('fullscreenMode'); if(isFullscreenMode){wp.data.dispatch('core/edit-post').toggleFeature('fullscreenMode');} });";
            }
            
            // Disable Welcome Guide
            if($this->wpbe_valid_key('wpbe_disable_gtb_welcome')){
                $scripts .= "jQuery(window).load(function(){ const welcomeGuide = wp.data.select('core/edit-post').isFeatureActive('welcomeGuide'); if(welcomeGuide){wp.data.dispatch('core/edit-post').toggleFeature('welcomeGuide');} });";
            }
            
            // Add the scripts inlined
            if($scripts){
                wp_add_inline_script('wp-blocks', $scripts);
            }
        }
        
        /**
         * Changes admin footer left sidehandle text
         * 
         * @since 5.2.0
         */
        public function wpbe_admin_footer_left(){ 
            return $this->wpbe_keys['wpbe_admin_footer_left']; 
        }

        /**
         * Changes admin footer right sidehandle text
         * 
         * @since 5.2.0
         */
        public function wpbe_admin_footer_right(){ 
            return $this->wpbe_keys['wpbe_admin_footer_right']; 
        }

        /**
         * Changes the system from adress in mails sent to users
         * 
         * @since 5.2.0
         */
        public function wpbe_wp_mail_from_adress($old){
            return $this->wpbe_keys['wpbe_wp_mail_from_adress'];
        }

        /**
         * Changes the system from name in mails sent to users
         * 
         * @since 5.2.0
         */
        public function wpbe_wp_mail_from_name($old){
            return $this->wpbe_keys['wpbe_wp_mail_from_name'];
        }

    } // End CLASS WPBE_Functions
} // End Class Check if exists

// In BaseClass constructor
WPBE_Functions::get_instance();